import objectdraw.*;
import java.awt.*;

/*
 * Example NestedSquares: a recursive implementation of an object
 * consisting of nested squares, complete with contains and move
 * methods to support dragging.
 *
 * Jim Teresco, The College of Saint Rose, CSC 252, Fall 2013
 *
 * $Id: NestedSquares.java 2230 2013-10-27 02:26:10Z terescoj $
 */

public class NestedSquares {

    private static final int SIZE_CHANGE = 4;
    
    // instance variables to keep track of the outermost
    // square and then NestedSquares inside
    private FramedRect outline;
    private NestedSquares rest;
    
    // a recursive implementation: we draw nested squares
    // by creating the outermost square, then calling
    // THIS CONSTRUCTOR to draw the rest, which are really
    // just another set of nested squares, just a little bit
    // smaller and a little bit down and to the right.
    public NestedSquares(double x, double y, int size, DrawingCanvas c) {
        
        // draw one        
        outline = new FramedRect(x, y, size, size, c);
        
        // draw more only if there are more to draw, otherwise
        // we remember rest as null (so we know the recursion
        // is over) -- if size is not greater than SIZE_CHANGE,
        // the next level would not exist, so we don't try to
        // draw it
        if (size > SIZE_CHANGE) {
            // draw the rest
            rest = new NestedSquares(x + SIZE_CHANGE/2, y+SIZE_CHANGE/2,
                size - SIZE_CHANGE, c);
        }
        else {
            // this will flag the end of the recursion
            rest = null;
        }
    }
    
    // contains is easy - we just check containment of the outline
    public boolean contains(Location point) {
        
        return outline.contains(point);
    }
    
    // move method will need to be recursive: we move the whole thing
    // by moving the outline, then moving the rest (using this method!)
    public void move(double dx, double dy) {
        
        outline.move(dx, dy);
        if (rest != null) {
            rest.move(dx, dy);
        }
    }
    
    // we will also be able to set the color
    public void setColor(Color c) {
        
        // we achieve this recursively
        outline.setColor(c);
        if (rest != null) {
            rest.setColor(c);
        }
    }
    
    // a recursive accessor method to count the number of nested squares
    public int squareCount() {
        
        // if this is the innermost, we have just the 1
        if (rest == null) return 1;
        // otherwise we have the one (the outline) plus whatever
        // number is in the rest
        return 1 + rest.squareCount();
    }
    
    // also compute the total perimeter of all of the nested squares
    public double totalPerimeter() {
        
        // first, compute the perimeter of the outermost square
        double outlinePerim = 4 * outline.getWidth();
        
        // again, if this is the innermost, we have just this one to consider
        if (rest == null) return outlinePerim;
        
        // otherwise, we have this one plus the smaller ones inside of rest
        return outlinePerim + rest.totalPerimeter();
    }
}
